import { AxiosError } from 'axios';
import { JsonApiErrorMetaType, JsonApiResponseErrors, JsonApiResponseNestedErrors } from '../types';
/**
 * Base class for server errors.
 */
export declare class JsonApiErrorBase extends Error {
    originalError: AxiosError<JsonApiResponseErrors>;
    _meta: JsonApiErrorMetaType;
    _detail: string;
    _title: string | undefined;
    _nestedErrors: JsonApiResponseNestedErrors;
    constructor(originalError: AxiosError<JsonApiResponseErrors>);
    /**
     * Response HTTP status.
     */
    get httpStatus(): number | undefined;
    /**
     * Error meta.
     */
    get meta(): JsonApiErrorMetaType;
    /**
     * A short, human-readable summary of the problem.
     */
    get title(): string | undefined;
    /**
     * A human-readable explanation specific to this occurrence of the problem.
     */
    get detail(): string;
    get requestPath(): string | undefined;
}
/**
 * Generic server error response.
 */
export declare class JsonApiError extends JsonApiErrorBase {
    constructor(originalError: AxiosError<JsonApiResponseErrors>);
}
export declare class BadRequestError extends JsonApiError {
    /**
     * Wrap a raw API error response.
     */
    constructor(originalError: AxiosError<JsonApiResponseErrors>);
    /**
     * Errors for every invalid field.
     */
    get nestedErrors(): JsonApiResponseNestedErrors;
}
/**
 * 401(Unauthorized) error.
 */
export declare class UnauthorizedError extends JsonApiError {
}
/**
 * 403(Forbidden) error.
 */
export declare class ForbiddenError extends JsonApiError {
}
/**
 * (404)The requested resource was not found.
 */
export declare class NotFoundError extends JsonApiError {
}
/**
 * The request could not be completed due to a conflict with the current state
 * of the target resource.
 */
export declare class ConflictError extends JsonApiError {
}
/**
 * 500(Internal server) error
 */
export declare class InternalServerError extends JsonApiError {
}
/**
 * Network error.
 */
export declare class NetworkError extends JsonApiError {
}
