import { AxiosResponse, RawAxiosResponseHeaders } from 'axios';
import { JsonApiDefaultMeta, JsonApiLinkFields, JsonApiResponseLinks } from './types';
import { JsonApiClient } from './json-api';
/**
 * API response wrapper.
 */
export declare class JsonApiResponse<T, U = JsonApiDefaultMeta> {
    private _raw;
    private _rawData;
    private _data;
    private _links;
    private _apiClient;
    private _isNeedRaw;
    private _withCredentials;
    private _meta;
    constructor(opts: {
        raw: AxiosResponse;
        isNeedRaw: boolean;
        apiClient: JsonApiClient;
        withCredentials: boolean;
    });
    get meta(): U;
    /**
     * Get raw response.
     */
    get rawResponse(): AxiosResponse;
    /**
     * Get request page limit.
     */
    get pageLimit(): number | undefined;
    /**
     * Get raw response data.
     */
    get rawData(): Record<string, unknown>;
    /**
     * Get response data.
     */
    get data(): T;
    /**
     * Get response HTTP status.
     */
    get status(): number;
    /**
     * Get response headers.
     */
    get headers(): RawAxiosResponseHeaders;
    /**
     * Get response links.
     */
    get links(): JsonApiResponseLinks;
    /**
     * Is response links exist.
     */
    get isLinksExist(): boolean;
    /**
     * Parses and unwraps response data.
     */
    private _parseResponse;
    private _createLink;
    fetchPage(page: JsonApiLinkFields): Promise<JsonApiResponse<T, U>>;
}
